#include "StdAfx.h"
#include ".\nstringresourceloader.h"

#define TIXML_USE_STL
#include "tinyxml.h"

#include "XMLUtil.h"
#include "NByteArray.h"
#include <atlpath.h>

static const TCHAR	LANG[] = _T("lang");
static const TCHAR	RESOURCE[] = _T("resource");
static const TCHAR	STRING[] = _T("string");
static const TCHAR	CONST_NODE[] = _T("const");
static const TCHAR	VALUE[] = _T("value");

/************************************************************************/
/*    CNStringResourceLoader                                                             
/************************************************************************/
CNStringResourceLoader::CNStringResourceLoader(void)
{	
}

CNStringResourceLoader::~CNStringResourceLoader(void)
{
	
}
BOOL CNStringResourceLoader::Init(CNPUnzipFile*	zipImage)
{
	__super::Init(zipImage);

	BOOL bSuccess = FALSE;
	
	m_ZipImage->GoToFirstFile();

	UnzipFileInfo fileInfo;
	char szCurrentFileName[_MAX_PATH+1];

	BOOL bNextFile = TRUE;
	while (bNextFile)
	{
		m_ZipImage->GetCurrentFileInfo(&fileInfo, szCurrentFileName, _MAX_PATH, NULL, 0, NULL, 0);
		if (strncmp("string", szCurrentFileName, strlen("binary")) == 0 &&
			strcmp(".xml", ATLPath::FindExtension(szCurrentFileName)) == 0)
		{
			LONG lFileSize = fileInfo.m_lUncompressedSize;
			if (lFileSize > 0)
			{
				CNByteArray bufArray;
				ReadCurrentZipFile(bufArray, lFileSize);

				if (bufArray.GetSize() > 0)
				{
					_ParseXML(bufArray.GetBuffer());
				}
			}
		}
		bNextFile = m_ZipImage->GoToNextFile();
	}
	m_ZipImage->GoToFirstFile();
	return bSuccess;
}
BOOL CNStringResourceLoader::Close()
{
	m_mapString.RemoveAll();
	return TRUE;
}
LPCTSTR	CNStringResourceLoader::GetString(DWORD dwID)
{
	POSITION pos = m_lstString.FindIndex(dwID-1);
	CString strKey = m_lstString.GetAt(pos);
	
	return GetString(strKey);
}
LPCTSTR CNStringResourceLoader::GetString(LPCTSTR pszcID)
{
	CString strKey = pszcID;
	CAtlMap< CString, CString, CStringElementTraits<CString>, CStringElementTraits<CString> >::CPair* pPair = m_mapString.Lookup(strKey);
	if (pPair == NULL)
	{
		//static LPCTSTR emptyString = _T("");
		return pszcID;
	}
	return pPair->m_value;
}
BOOL CNStringResourceLoader::_ParseXML(const LPBYTE pBuf)
{
	USES_CONVERSION;

	BOOL bSuccess = FALSE;
	TiXmlDocument document;
	document.Parse(reinterpret_cast<LPCSTR>(pBuf));
	TiXmlElement* pRoot = document.FirstChildElement(T2A(RESOURCE));
	if (!pRoot) return FALSE;
	TiXmlElement* pLangNode = pRoot->FirstChildElement(T2A(LANG));

	static const int MAX_ID = 128;
	static const int MAX_VALUE = 1024;

	TCHAR	szID[MAX_ID+1] = {0};
	TCHAR	szValue[MAX_VALUE+1] = {0};
	TiXmlElement* pStringNode = pRoot->FirstChildElement(T2A(STRING));
	while (pStringNode)
	{
		XMLUtil::GetAttrValue(pStringNode, T2A(CONST_NODE), szID, MAX_ID);
		//XMLUtil::GetValue(pStringNode, T2A(VALUE), szValue, MAX_VALUE);
		XMLUtil::GetValue(pStringNode, szValue, MAX_VALUE);


		m_mapString.SetAt(CString(szID), CString(szValue));
		m_lstString.AddTail(CString(szID));

		pStringNode = pStringNode->NextSiblingElement(T2A(STRING));
	}
	return bSuccess;
}
BOOL CNStringResourceLoader::GetStringList(CAtlMap< CString, CString, CStringElementTraits<CString>, CStringElementTraits<CString> >& lstString)
{
	POSITION pos = m_mapString.GetStartPosition();
	while (pos)
	{
		CAtlMap< CString, CString, CStringElementTraits<CString>, CStringElementTraits<CString> >::CPair* pPair = m_mapString.GetNext(pos);
		lstString.SetAt(pPair->m_key, pPair->m_value);
	}
	return TRUE;
}